searchNodes=[{"type":"module","title":"erlperf","doc":"Convenience APIs for benchmarking. This module implements following benchmarking modes: Continuous mode Timed (low overhead) mode Concurrency estimation (squeeze) mode Continuous mode This is the default mode. Separate erlperf_job is started for each benchmark, iterating supplied runner in a tight loop, bumping a counter for each iteration of each worker. erlperf reads this counter every second (or sample_duration ), calculating the difference between current and previous value. This difference is called a sample . By default, erlperf collects 3 samples and stops, reporting the average. To give an example, if your function runs for 20 milliseconds, erlperf may capture samples with 48, 52 and 50 iterations. The average would be 50. This approach works well for CPU-bound calculations, but may produce unexpected results for slow functions taking longer than sample duration. For example, timer:sleep(2000) with default settings yields zero throughput. You can change the sample duration and the number of samples to take to avoid that. Timed mode In this mode erlperf loops your code a specified amount of times, measuring how long it took to complete. It is essentially what timer:tc/3 does. This mode has slightly less overhead compared to continuous mode. This difference may be significant if you’re profiling low-level ERTS primitives. This mode does not support concurrency setting (concurrency locked to 1). Concurrency estimation mode In this mode erlperf attempts to estimate how concurrent the supplied runner code is. The run consists of multiple passes, increasing concurrency with each pass, and stopping when total throughput is no longer growing. This mode proves useful to find concurrency bottlenecks. For example, some functions may have limited throughput because they execute remote calls served by a single process. See benchmark/3 for the detailed description.","ref":"erlperf.html"},{"type":"function","title":"erlperf.benchmark/3","doc":"Generic benchmarking suite, accepting multiple code maps, modes and options. Codes contain a list of code versions. Every element is a separate job that runs in parallel with all other jobs. Same RunOptions are applied to all jobs. ConcurrencyTestOpts specifies options for concurrency estimation mode. Passing undefined results in a continuous or a timed run. It is not supported to run multiple jobs while doing a concurrency estimation run. Concurrency estimation run consists of multiple passes. First pass is done with a min number of workers, subsequent passes are increasing concurrency by 1, until max concurrency is reached, or total job iterations stop growing for threshold consecutive passes. To give an example, if your code is not concurrent at all, and you try to benchmark it with threshold set to 3, there will be 4 passes in total: first with a single worker, then 3 more, demonstrating no throughput growth. In this mode, job is started once before the first pass. Subsequent passes only change the concurrency. All other options passed in RunOptions are honoured. So, if you set samples to 30, keeping default duration of a second, every single pass will last for 30 seconds.","ref":"erlperf.html#benchmark/3"},{"type":"function","title":"erlperf.compare/2","doc":"Comparison run: benchmark multiple jobs at the same time. A job is defined by either erlperf_job:code_map() , or just the runner callable . Example comparing rand:uniform/0 %% performance to rand:mwc59/1 : ( erlperf @ ubuntu22 ) 7 &gt; erlperf : compare ( [ { rand , uniform , [ ] } , \#{ runner =&gt; &quot;run(X) -&gt; rand:mwc59(X).&quot; , init_runner =&gt; { rand , mwc59_seed , [ ] } } ] , \#{ } ) . [ 14823854 , 134121999 ] See benchmark/3 for RunOptions definition and return values.","ref":"erlperf.html#compare/2"},{"type":"function","title":"erlperf.run/1","doc":"Runs a single benchmark for 3 seconds, returns average number of iterations per second. Accepts either a full erlperf_job:code_map() , or just the runner callable .","ref":"erlperf.html#run/1"},{"type":"function","title":"erlperf.run/2","doc":"Runs a single benchmark job, returns average number of iterations per second, or a full report. Accepts either a full erlperf_job:code_map() , or just the runner callable . Equivalent of returning the first result of run([Code], RunOptions) .","ref":"erlperf.html#run/2"},{"type":"function","title":"erlperf.run/3","doc":"Concurrency estimation run, or an alias for quick benchmarking of an MFA tuple. Attempt to find concurrency characteristics of the runner code, see benchmark/3 for a detailed description. Accepts either a full erlperf_job:code_map() , or just the runner callable . When Module and Function are atoms, and Args is a list, this call is equivalent of run(Module, Function, Args) .","ref":"erlperf.html#run/3"},{"type":"function","title":"erlperf.start/2","doc":"Starts a new supervised job with the specified concurrency. Requires erlperf application to be running. Returns job controller process identifier. This function is designed for distributed benchmarking, when jobs are started in different nodes, and monitored via erlperf_cluster_monitor .","ref":"erlperf.html#start/2"},{"type":"function","title":"erlperf.time/2","doc":"Timed benchmarking mode. Iterates the runner code Count times and returns elapsed time in microseconds. This method has lower overhead compared to continuous benchmarking. It is not supported to run multiple workers in this mode.","ref":"erlperf.html#time/2"},{"type":"type","title":"erlperf.code/0","doc":"Convenience type used in run/1,2,3 and compare/2 .","ref":"erlperf.html#t:code/0"},{"type":"type","title":"erlperf.concurrency_result/0","doc":"Basic concurrency estimation report Only the highest throughput run is reported. Concurrency contains the number of concurrently running workers when the best result is achieved.","ref":"erlperf.html#t:concurrency_result/0"},{"type":"type","title":"erlperf.concurrency_test/0","doc":"Concurrency estimation mode options. min : initial number of workers, default is 1 max : maximum number of workers, defaults to erlang:system_info(process_limit) - 1000 threshold : stop concurrency run when adding this amount of workers does not result in further total throughput increase. Default is 3","ref":"erlperf.html#t:concurrency_test/0"},{"type":"type","title":"erlperf.concurrency_test_result/0","doc":"Concurrency estimation mode result Extended report contains results for all runs, starting from the minimum number of workers, to the highest throughput detected, plus up to threshold more.","ref":"erlperf.html#t:concurrency_test_result/0"},{"type":"type","title":"erlperf.isolation/0","doc":"Node isolation settings. Currently, host selection is not supported.","ref":"erlperf.html#t:isolation/0"},{"type":"type","title":"erlperf.report/0","doc":"Full benchmark report, containing all collected samples and statistics mode : benchmark run mode result : benchmark result. Concurrency estimation mode contains the best result (with the highest average throughput recorded) code : original code run_options : full set of options, with all defaults filled in system : information about the system benchmark is running on history : returned only for concurrency estimation mode, contains a list of all runs with their results concurrency_options : returned only for concurrency estimation mode, with all defaults filled in sleep : method used for waiting for a specified amount of time. Normally set to sleep , but may be reported as busy_wait if erlperf scheduling is impacted by lock contention or another problem preventing it from using precise timing","ref":"erlperf.html#t:report/0"},{"type":"type","title":"erlperf.run_options/0","doc":"Benchmarking mode selection and parameters of the benchmark run. concurrency : number of workers to run, applies only for the continuous benchmarking mode cv : coefficient of variation. Acceptable standard deviation for the test to conclude. Not applicable for timed mode. When the value is set, benchmark will continue running until standard deviation for the last collected samples divided by average value (arithmetic mean) is smaller than cv specified. isolation : request separate Erlang VM instance for each job. Some benchmarks may lead to internal VM structures corruption, or change global structures affecting other benchmarks when running in the same VM. host sub-option is currently ignored. priority : sets the job controller process priority (defaults to high ). Running with normal or lower priority may prevent the controller from timely starting ot stopping workers. report : applies only for continuous mode. basic report contains only the average value. Specify extended to get the list of actual samples, to calculate exotic statistics. Pass full to receive full report, including benchmark settings and extra statistics for continuous mode - minimum, maximum, average, median and 99th percentile (more metrics may be added in future releases) samples : number of measurements to take. Default is 3. For continuous mode it results in a 3 second run, when sample_duration is set to default 1000 ms. sample_duration : time, milliseconds, between taking iteration counter samples. Multiplied by samples , this parameter defines the total benchmark run duration. Default is 1000 ms. Passing {timed, Counter}` engages timed mode with `Counter iterations taken samples times warmup : how many extra samples are collected and discarded at the beginning of the continuous run","ref":"erlperf.html#t:run_options/0"},{"type":"type","title":"erlperf.run_result/0","doc":"Benchmark results. For continuous mode, an average (arithmetic mean) of the collected samples, or a list of all samples collected. Timed mode returns elapsed time (microseconds).","ref":"erlperf.html#t:run_result/0"},{"type":"type","title":"erlperf.run_statistics/0","doc":"Results reported by a single benchmark run. best : highest throughput for continuous mode, or lowest time for timed worst : lowest throughput, or highest time average : arithmetic mean, iterations for continuous mode and microseconds for timed stddev : standard deviation median : median (50% percentile) p99 : 99th percentile samples : raw samples from the run, monotonic counter for continuous mode, and times measured for timed run time : total benchmark duration (us), may exceed sample * sample_duration when cv is specified and results are not immediately stable iteration_time : approximate single iteration time (of one runner)","ref":"erlperf.html#t:run_statistics/0"},{"type":"type","title":"erlperf.system_information/0","doc":"System information, as returned by erlang:system_info/1 May also contain CPU model name on supported operating systems.","ref":"erlperf.html#t:system_information/0"},{"type":"module","title":"erlperf_cli","doc":"Command line interface adapter. Exports functions to format erlperf:benchmark/3 output in the same way as command line interface. Example: # ! / usr / bin / env escript %%! +pc unicode -pa /home/max-au/git/max-au/erlperf/_build/default/lib/erlperf/ebin - mode ( compile ) . main ( _ ) -&gt; Report = erlperf : benchmark ( [ \#{ runner =&gt; fun ( ) -&gt; rand : uniform ( 10 ) end } , \#{ runner =&gt; { rand , mwc59 , [ 1 ] } } ] , \#{ report =&gt; full } , undefined ) , Out = erlperf_cli : format ( Report , \#{ format =&gt; extended , viewport_width =&gt; 120 } ) , io : format ( Out ) , halt ( 0 ) . Running the script produces following output: $ . / bench Code || Samples Avg StdDev Median P99 Iteration Rel { rand , mwc59 , [ 1 ] } 1 3 80515 Ki 0.59 % 80249 Ki 81067 Ki 12 ns 100% # Fun &lt; bench__escript__1674__432325__319865__16 . 0 . 1 3 15761 Ki 0.48 % 15726 Ki 15847 Ki 63 ns 20%","ref":"erlperf_cli.html"},{"type":"function","title":"erlperf_cli.format/2","doc":"Formats result produced by erlperf:benchmark/3 . Requires full report. Does not accept basic or extended variants.","ref":"erlperf_cli.html#format/2"},{"type":"type","title":"erlperf_cli.format_options/0","doc":"Defines text report format. format : basic : default format containing only average throughput per sample_duration and average runner runtime extended : includes median, p99 and other metrics(default for 10 and more samples) full : includes system information in addition to extended output See overview for the detailed description viewport_width : how wide the report can be, defaults to io:columns/0 . Falls back to 80 when terminal does not report width.","ref":"erlperf_cli.html#t:format_options/0"},{"type":"module","title":"erlperf_cluster_monitor","doc":"Logs monitoring events for the entire cluster, to file or device. Requires erlperf_history service running, fails otherwise. Uses completely different to erlperf_monitor approach; instead of waiting for new samples to come, cluster monitor just outputs existing samples periodically. Example primary node: rebar3 shell -- sname primary ( primary @ ubuntu22 ) 1 &gt; erlperf_history : start_link ( ) . { ok , &lt; 0.211 . 0 &gt; } ( primary @ ubuntu22 ) 2 &gt; erlperf_cluster_monitor : start_link ( ) . { ok , &lt; 0.216 . 0 &gt; } Example benchmarking node: rebar3 shell -- sname bench1 ( bench1 @ ubuntu22 ) 1 &gt; net_kernel : connect_node ( &#39;primary@ubuntu22&#39; ) . true ( bench1 @ ubuntu22 ) 2 &gt; erlperf : run ( rand , uniform , [ ] ) . As soon as the new benchmarking jon on the node bench is started, it is reported in the cluster monitoring output.","ref":"erlperf_cluster_monitor.html"},{"type":"function","title":"erlperf_cluster_monitor.start_link/0","doc":"Equivalent to start_link(erlang:group_leader(), 1000, undefined) .","ref":"erlperf_cluster_monitor.html#start_link/0"},{"type":"function","title":"erlperf_cluster_monitor.start_link/3","doc":"Starts cluster-wide monitor process, and links it to the caller. Intended to be used in a supervisor ChildSpec , making the process a part of the supervision tree. IntervalMs specifies time, in milliseconds, between output handler invocations. Fields specifies the list of field names to report, and the order in which columns are printed. see erlperf_monitor:monitor_sample() for options. Passing undefined prints all columns known by this version of erlperf .","ref":"erlperf_cluster_monitor.html#start_link/3"},{"type":"type","title":"erlperf_cluster_monitor.handler/0","doc":"Specifies monitoring output device. It could be an output io:device() (such as erlang:group_leader/0 , user or standard_error ), a file name, or a {Module, Function, UserState} tuple. In the latter case, instead of printing, cluster monitor calls the specified function, which must have arity of 2, accepting filtered erlperf_monitor:monitor_sample() as the first argument, and Userstate as the second, returning next UserState .","ref":"erlperf_cluster_monitor.html#t:handler/0"},{"type":"module","title":"erlperf_file_log","doc":"Prints monitoring reports produced by erlperf_monitor to file or an output device. When the server starts up, it joins {erlperf, Node} process group in the erlperf scope. If erlperf_monitor is also running in the same node, reports are printed to the specified device or file. See erlperf_monitor for description of the monitoring report. erlperf leverages this service for verbose output during benchmarking.","ref":"erlperf_file_log.html"},{"type":"function","title":"erlperf_file_log.start_link/0","doc":"Equivalent to start_link(erlang:group_leader()) .","ref":"erlperf_file_log.html#start_link/0"},{"type":"function","title":"erlperf_file_log.start_link/1","doc":"Starts the file log process.","ref":"erlperf_file_log.html#start_link/1"},{"type":"module","title":"erlperf_history","doc":"Collects, accumulates &amp; filters cluster-wide monitoring events. Essentially a simple in-memory database for quick cluster overview. History server helps to collect monitoring reports from multiple nodes of a single Erlang cluster. Example setup: single primary node running erlperf_history and erlperf_cluster_monitor listens to reports sent by several more nodes in a cluster, running continuous benchmarking jobs. Nodes may run the same Erlang code, but using different hardware or OS version. Or, conversely, same hardware and OS, but variants of Erlang code. See erlperf_cluster_monitor for a code sample.","ref":"erlperf_history.html"},{"type":"function","title":"erlperf_history.get/1","doc":"Returns cluster history. Returns all reports since From timestamp to now, sorted by timestamp. From is wall clock time, in milliseconds (e.g. os:system_time(millisecond) ).","ref":"erlperf_history.html#get/1"},{"type":"function","title":"erlperf_history.get/2","doc":"Returns cluster history reports between From and To (inclusive). From and To are wall clock time, in milliseconds (e.g. os:system_time(millisecond) ).","ref":"erlperf_history.html#get/2"},{"type":"function","title":"erlperf_history.start_link/0","doc":"Equivalent to start_link(120000) .","ref":"erlperf_history.html#start_link/0"},{"type":"function","title":"erlperf_history.start_link/1","doc":"Starts the history server and links it to the calling process. Designed for use as a part of a supervision tree. Duration is time (in milliseconds), how long to keep the reports for. Older reports are discarded.","ref":"erlperf_history.html#start_link/1"},{"type":"module","title":"erlperf_job","doc":"Job is an instance of a benchmark. Every job has a corresponding temporary Erlang module generated. Use source/1 to get the source code of the generated module. The structure of this code is an implementation detail and may change between releases. Job controls how many workers are executing runner code in a tight loop. It does not restart a failing worker, user must ensure proper error handing and reporting. If a worker process crashes, standard CRASH REPORT message is printed to the log (console). Job accepts a code_map() containing at least a runner function definition. See callable() for accepted function definitions. Different callable forms have different performance overhead. Overhead can be measured with erlperf:compare/2 : erlang erlperf : compare ( [ \#{ runner =&gt; fun ( V ) -&gt; rand : mwc59 ( V ) end , init_runner =&gt; { rand , mwc59_seed , [ ] } } , \#{ runner =&gt; &quot;run(V) -&gt; rand:mwc59(V).&quot; , init_runner =&gt; { rand , mwc59_seed , [ ] } } ] , \#{ } ) . [ 4371541 , 131460130 ] In the example above, callable defined as fun is 30 times slower than the code compiled from the source. The difference is caused by the Erlang Runtime implementation, where indirect calls via fun are considerably more expensive. As a rule of thumb, source code provides the smallest overhead, followed by MFA tuples. You can mix &amp; match various definition styles. In the example below, init/0 starts an extra pg scope, done/0 stops it, and init_runner/1 takes the total heap size of pg scope controller to pass it to the runner/1 . erlang erlperf_job : start_link ( \#{ runner =&gt; &quot;run(Max) -&gt; rand:uniform(Max).&quot; , init =&gt; { pg , start_link , [ scope ] } , init_runner =&gt; fun ( { ok , Pid } ) -&gt; { total_heap_size , THS } = erlang : process_info ( Pid , total_heap_size ) , THS end , done =&gt; fun ( { ok , Pid } ) -&gt; gen_server : stop ( Pid ) end } ) . Same example defined with just the source code: erlang erlperf_job : start_link ( \#{ runner =&gt; &quot;runner(Max) -&gt; rand:uniform(Max).&quot; , init =&gt; &quot;init() -&gt; pg:start_link().&quot; , init_runner =&gt; &quot;init_runner({ok, Pid}) -&gt; {total_heap_size, THS} = erlang:process_info(Pid, total_heap_size), THS.&quot; , done =&gt; &quot;done({ok, Pid}) -&gt; gen_server:stop(Pid).&quot; } ) . Runner function Runner function represents code that is run in the tight loop, counting iterations aggregated between all workers. To give an example, benchmarking a function that takes exactly a millisecond to execute, with 2 workers, for 2 seconds, will result in 4000 iterations in total. This would be the value returned by sample/1 . Runner definition can accept zero, one or two arguments. runner/0 ignores the value returned by init_runner/0,1. runner/1 accepts the value returned by init_runner/0,1. It is an error to define runner/1 without init_runner/0,1 defined. This example prints &quot;0&quot; in a tight loop, measuring io:format/2 performance: erlang \#{ runner =&gt; &quot;run(Init) -&gt; io:format(\\&quot; ~b ~n \\&quot;, [Init]).&quot; , init_runner =&gt; &quot;0.&quot; } runner/2 adds second argument, accumulator, initially set to the value returned by init_runner/0,1. Subsequent invocations receive value returned by the previous runner invocation. Example: erlang \#{ runner =&gt; &quot;run(Init, Acc) -&gt; io:format(\\&quot; ~b ~n \\&quot;, [Init + Acc]), Acc + 1.&quot; , init_runner =&gt; &quot;0.&quot; } Running this benchmark prints monotonically increasing numbers. This may be useful to test stateful functions, for example, fast Random Number Generators introduced in OTP 25: bash . / erlperf -- init_runner &#39;rand:mwc59_seed().&#39; &#39;run(_, Cur) -&gt; rand:mwc59(Cur).&#39; Code || QPS Time run ( _ , Cur ) -&gt; rand : mwc59 ( Cur ) . 1 123 Mi 8 ns Common Test usage Example using erlperf_job directly, as a part of Common Test test case: erlang benchmark_rand ( Config ) when is_list ( Config ) -&gt; %% run timer:sleep(1000) for 5 second, 4 runners { ok , Job } = erlperf_job : start_link ( \#{ runner =&gt; { timer , sleep , [ 1000 ] } } ) , Handle = erlperf_job : handle ( Job ) , ok = erlperf_job : set_concurrency ( Job , 4 ) , %% 4 runner instances InitialIterations = erlperf_job : sample ( Handle ) , timer : sleep ( 5000 ) , IterationsIn5Sec = erlperf_job : sample ( Handle ) - InitialIterations , erlperf_job : request_stop ( Job ) , %% use gen:stop(Job) for synchronous call %% expect at least 16 iterations (and up to 20) ? assert ( IterationsIn5Sec &gt;= 16 , { too_slow , IterationsIn5Sec } ) , ? assert ( IterationsIn5Sec =&lt; 20 , { too_fast , IterationsIn5Sec } ) .","ref":"erlperf_job.html"},{"type":"function","title":"erlperf_job.concurrency/1","doc":"Returns the number of concurrently running workers for this job. This number may be lower than the amount requested by set_concurrency/2 if workers crash.","ref":"erlperf_job.html#concurrency/1"},{"type":"function","title":"erlperf_job.handle/1","doc":"Returns the sampling handle for the job. The returned value is opaque, and is an implementation detail, do not use it in any quality other than passing to sample/1 .","ref":"erlperf_job.html#handle/1"},{"type":"function","title":"erlperf_job.measure/2","doc":"Run the timed mode benchmark for a job, similar to timer:tc/3 . Executes the runner SampleCount times. Returns time in microseconds. Has less overhead compared to continuous benchmarking, therefore can be used even for very fast functions.","ref":"erlperf_job.html#measure/2"},{"type":"function","title":"erlperf_job.request_stop/1","doc":"Requests this job to stop. Job is stopped asynchronously. Caller should monitor the job process to find out when the job actually stopped.","ref":"erlperf_job.html#request_stop/1"},{"type":"function","title":"erlperf_job.sample/1","doc":"Returns the current iteration counter. The iteration counter (sample) monotonically grows by 1 every time the runner function is called (without waiting for it to return, so a function that unconditionally crashes still generates a counter of 1).","ref":"erlperf_job.html#sample/1"},{"type":"function","title":"erlperf_job.set_concurrency/2","doc":"Sets the number of concurrently running workers for this job. Does not reset counting. May never return if init_runner hangs and does not return control the the job. Concurrency : number of processes to run. It can be higher than the current count (making the job to start more workers), or lower, making the job to stop some. Workers that crashes are not restarted automatically.","ref":"erlperf_job.html#set_concurrency/2"},{"type":"function","title":"erlperf_job.set_priority/2","doc":"Sets job process priority when there are workers running. Worker processes may utilise all schedulers, making job process to lose control over starting and stopping workers. By default, job process sets 'high' priority when there are any workers running. Returns the previous setting. This function must be called before set_concurrency/2 , otherwise it has no effect until all workers are stopped, and then restarted.","ref":"erlperf_job.html#set_priority/2"},{"type":"function","title":"erlperf_job.source/1","doc":"Returns the source code generated from the code map, or for a running job.","ref":"erlperf_job.html#source/1"},{"type":"function","title":"erlperf_job.start/1","doc":"Starts the benchmark job. Job starts with no workers, use set_concurrency/2 to start workers.","ref":"erlperf_job.html#start/1"},{"type":"function","title":"erlperf_job.start_link/1","doc":"Starts the job and links it to caller. Job starts with no workers, use set_concurrency/2 to start workers.","ref":"erlperf_job.html#start_link/1"},{"type":"type","title":"erlperf_job.callable/0","doc":"Function definition to use as a runner, init, done or init_runner. string(). Erlang code ending with . (period). Example, zero arity: &quot;runner() -&gt; timer:sleep(1).&quot; , arity one: &quot;runner(T) -&gt; timer:sleep(T).&quot; , arity two: &quot;runner(Init, Acc) -&gt; Acc + Init.&quot; . It is allowed to omit the header for zero arity function, so it becomes &quot;timer:sleep(1).&quot; fun() function accepting no arguments, example: fun() -&gt; timer:sleep(1000) end fun(term()) -&gt; term() function accepting one argument, example: fun(Time) -&gt; timer:sleep(Time) end fun(term(), term()) -&gt; term() function accepting two arguments, example: fun() -&gt; timer:sleep(1000) end mfargs() tuple accepted by erlang:apply/3 . Example: {rand, uniform, [10]} [mfargs()] list of MFA tuples, example: [{rand, uniform, [10]}] . This functionality is experimental, and only used to replay a recorded calls list. May not be supported in future releases.","ref":"erlperf_job.html#t:callable/0"},{"type":"type","title":"erlperf_job.code_map/0","doc":"Code map contains definitions for: init/0 - called once when starting the job for the first time. The call is made in the context of the job controller. It is guaranteed to run through the entire benchmark job. So if your benchmark needs to create additional resources - ETS tables, or linked processes, like extra pg scopes, - init/0 is a good choice. If init/0 fails, the entire job startup fails init_runner/0,1 - called when the job starts a new worker. init_runner/1 accepts the value returned by init/0. It is an error to omit init/0 if init_runner/1 is defined. It is allowed to have init_runner/0 when init/0 exists. The call to init_runner is made in the context of the worker process, so you can initialise process-local values (e.g. process dictionary) runner/0,1,2 defines the function that will be called in a tight loop. See Runner Function for overview of a runner function variants. done/0,1 - called when the job terminates, to clean up any resources that are not destroyed automatically. done/0 accepts the return of init/0. Call is made in the context of the job controller","ref":"erlperf_job.html#t:code_map/0"},{"type":"type","title":"erlperf_job.exec/0","doc":"","ref":"erlperf_job.html#t:exec/0"},{"type":"opaque","title":"erlperf_job.handle/0","doc":"","ref":"erlperf_job.html#t:handle/0"},{"type":"type","title":"erlperf_job.mfargs/0","doc":"Module , Function , Args accepted by erlang:apply/3 .","ref":"erlperf_job.html#t:mfargs/0"},{"type":"type","title":"erlperf_job.server_ref/0","doc":"","ref":"erlperf_job.html#t:server_ref/0"},{"type":"type","title":"erlperf_job.state/0","doc":"","ref":"erlperf_job.html#t:state/0"},{"type":"module","title":"erlperf_monitor","doc":"System monitor: reports scheduler, RAM, and benchmarks. Monitor is started by default when erlperf starts as an application. Monitor is not started for ad-hoc benchmarking (e.g. command-line, unless verbose logging is requested). When started, the monitor provides periodic reports about Erlang VM state, and registered jobs performance. The reports are sent to all processes that joined {erlperf_monitor, Node} or cluster_monitor process group in erlperf scope. Reports can be received by any process, even the shell. Run the following example in rebar3 shell of erlperf : ( erlperf @ ubuntu22 ) 1 &gt; ok = pg : join ( erlperf , cluster_monitor , self ( ) ) . ok ( erlperf @ ubuntu22 ) 2 &gt; erlperf : run ( rand , uniform , [ ] ) . 14976933 ( erlperf @ ubuntu22 ) 4 &gt; flush ( ) . Shell got { erlperf @ ubuntu22 , \#{ dcpu =&gt; 0.0 , dio =&gt; 6.42619095979426e-4 , ets =&gt; 44 , jobs =&gt; [ ] , memory_binary =&gt; 928408 , memory_ets =&gt; 978056 , memory_processes =&gt; 8603392 , memory_total =&gt; 34952096 , ports =&gt; 5 , processes =&gt; 95 , sched_util =&gt; 0.013187335960637163 , Note that the monitor may report differently from the benchmark run results. It is running with lower priority and may be significantly affected by scheduler starvation, timing issues etc..","ref":"erlperf_monitor.html"},{"type":"function","title":"erlperf_monitor.register/3","doc":"Registers an erlperf_job to monitor. Running monitor queries every registered job, adding the number of iterations performed by all workers of that job to the report. This API is intended to be used by erlperf_job to enable VM monitoring while benchmarking. Job specifies job process identifier, it is only used to detect when job is stopped, to stop reporting counters for that job. Handle is the sampling handle, see erlperf_job:handle/1 . Initial value should be provided when an existing job is registered, to avoid reporting accumulated counter value in the first report for that job. Always return ok , even when monitor is not running.","ref":"erlperf_monitor.html#register/3"},{"type":"function","title":"erlperf_monitor.start/0","doc":"Equivalent to start(\#{interval =&gt; 1000}) .","ref":"erlperf_monitor.html#start/0"},{"type":"function","title":"erlperf_monitor.start/1","doc":"Starts the monitor. Options are used to change the monitor behaviour. interval : time, in milliseconds, to wait between sample collection","ref":"erlperf_monitor.html#start/1"},{"type":"function","title":"erlperf_monitor.start_link/0","doc":"Equivalent to start_link(\#{interval =&gt; 1000}) .","ref":"erlperf_monitor.html#start_link/0"},{"type":"function","title":"erlperf_monitor.start_link/1","doc":"Starts the monitor and links it to the current process. See start/1 for options description.","ref":"erlperf_monitor.html#start_link/1"},{"type":"function","title":"erlperf_monitor.unregister/1","doc":"Removes the job from monitoring. Stops reporting this job performance. Job is the process identifier of the job.","ref":"erlperf_monitor.html#unregister/1"},{"type":"type","title":"erlperf_monitor.monitor_sample/0","doc":"Monitoring report time : timestamp when the report is generates, wall clock, milliseconds node : originating Erlang node name sched_util : normal scheduler utilisation, percentage. See scheduler:utilization/1 dcpu : dirty CPU scheduler utilisation, percentage. dio : dirty IO scheduler utilisation, percentage processes : number of processes in the VM. ports : number of ports in the VM. ets : number of ETS tables created in the VM. memory_total : total VM memory usage, see erlang:memory/1 . memory_processes : processes memory usage, see erlang:system_info/1 . memory_binary : binary memory usage. memory_ets : ETS memory usage. jobs : a map of job process identifier to the iterations surplus since last sample. If the sampling interval is default 1 second, the value of the map is &quot;requests/queries per second&quot; (RPS/QPS).","ref":"erlperf_monitor.html#t:monitor_sample/0"},{"type":"type","title":"erlperf_monitor.start_options/0","doc":"Monitor startup options interval : monitoring interval, 1000 ms by default","ref":"erlperf_monitor.html#t:start_options/0"},{"type":"extras","title":"Overview","doc":"Erlang Performance &amp; Benchmarking Suite. Simple way to say &quot;this code is faster than that one&quot;. See CLI reference and detailed API reference for erlperf and erlperf_job modules. Build (tested with OTP 23, 24, 25): $ rebar3 as prod escriptize","ref":"readme.html"},{"type":"extras","title":"Overview - Quick start: command line","doc":"Beware of the shell escaping your code in an unpredictable way! Run a single process iterating rand:uniform/0 in a tight loop for 3 seconds, printing average iterations per second (~17 millions) and an average time to run a single iteration (57 ns). $ ./erlperf &#39;rand:uniform().&#39; Code || QPS Time rand:uniform(). 1 17266 Ki 57 ns Run four processes doing this same concurrently. $ ./erlperf &#39;rand:uniform().&#39; -c 4 Code || QPS Time rand:uniform(). 4 53893 Ki 74 ns Benchmark rand:uniform/0 vs crypto:strong_rand_bytes/1 for 10 seconds, adding an extra second to warm up the algorithms. $ ./erlperf &#39;rand:uniform().&#39; &#39;crypto:strong_rand_bytes(2).&#39; --samples 10 --warmup 1 Code || Samples Avg StdDev Median P99 Iteration Rel rand:uniform(). 1 10 16611 Ki 0.20% 16614 Ki 16664 Ki 60 ns 100% crypto:strong_rand_bytes(2). 1 10 1804 Ki 0.79% 1797 Ki 1829 Ki 554 ns 11% Run a function passing the state into the next iteration. This code demonstrates performance difference between rand:uniform_s/1 with state passed explicitly, and rand:uniform/1 reading state from the process dictionary. $ ./erlperf &#39;r(_, S) -&gt; {_, N} = rand:uniform_s(S), N.&#39; --init_runner &#39;rand:seed(exsss).&#39; \\ &#39;r() -&gt; rand:uniform().&#39; Code || QPS Time Rel r(_, S) -&gt; {_, N} = rand:uniform_s(S), N. 1 26180 Ki 38 ns 100% r() -&gt; rand:uniform(). 1 16958 Ki 58 ns 65% Estimate ./erlperf 'application_controller:is_running(kernel). concurrency characteristics. This function is implemented as gen_server:call , and all calculations are done in a single process. It is still possible to squeeze a bit more from a single process by putting work into the queue from multiple runners. $ ./erlperf &#39;application_controller:is_running(kernel).&#39; --squeeze Code || QPS Time application_controller:is_running(kernel). 3 1189 Ki 2524 ns $ ./erlperf &#39;persistent_term:put(atom, &quot;string&quot;).&#39; -q Code || QPS Time persistent_term:put(atom, &quot;string&quot;). 1 8882 Ki 112 ns Start a server ( pg scope in this example), use it in benchmark, and shut down after. $ ./erlperf &#39;pg:join(scope, group, self()), pg:leave(scope, group, self()).&#39; \\ --init &#39;pg:start_link(scope).&#39; --done &#39;gen_server:stop(scope).&#39; Code || QPS Time pg:join(scope, group, self()), pg:leave(scope, group, self()). 1 336 Ki 2976 ns Run the same code with different arguments, returned from init_runner function. Note the trick of adding extra spaces in the source code to know which code is where. $ ./erlperf &#39;runner(X) -&gt; timer:sleep(X).&#39; --init_runner &#39;1.&#39; \\ &#39; runner(X) -&gt; timer:sleep(X).&#39; --init_runner &#39;2.&#39; Code || QPS Time Rel runner(X) -&gt; timer:sleep(X). 1 500 2001 us 100% runner(X) -&gt; timer:sleep(X). 1 333 3001 us 67% Determine how many times a process can join/leave pg2 group on a single node (requires OTP 23 or older, as pg2 is removed in later versions). $ ./erlperf &#39;ok = pg2:join(g, self()), ok = pg2:leave(g, self()).&#39; --init &#39;pg2:create(g).&#39; Code || QPS Time ok = pg2:join(g, self()), ok = pg2:leave(g, self()). 1 64021 15619 ns Compare pg with pg2 running in a 3-node cluster. Note the -i argument spawning an isolated extra Erlang VM for each benchmark. ./erlperf &#39;ok = pg2:join(g, self()), ok = pg2:leave(g, self()).&#39; --init &#39;pg2:create(g).&#39; \\ &#39;ok = pg:join(g, self()), ok = pg:leave(g, self()).&#39; --init &#39;pg:start(pg).&#39; -i Code || QPS Time Rel ok = pg:join(g, self()), ok = pg:leave(g, self()). 1 241 Ki 4147 ns 100% ok = pg2:join(g, self()), ok = pg2:leave(g, self()). 1 1415 707 us 0% Watch the progress of your test running ( -v option) with extra information: scheduler utilisation, dirty CPU &amp; IO schedulers, number of running processes, ports, ETS tables, and memory consumption. Last column is the job throughput. When there are multiple jobs, multiple columns are printed. Test will continue until adding 8 more workers ( -t 8 ) does not increase total throughput. $ ./erlperf &#39;rand:uniform().&#39; -q -v -t 8 YYYY-MM-DDTHH:MM:SS-oo:oo Sched DCPU DIO Procs Ports ETS Mem Total Mem Proc Mem Bin Mem ETS &lt;0.84.0&gt; 2023-01-22T11:02:51-08:00 6.12 0.00 0.20 46 2 21 24737 Kb 4703 Kb 191 Kb 471 Kb 14798 Ki 2023-01-22T11:02:52-08:00 6.31 0.00 0.00 46 2 21 25105 Kb 5565 Kb 218 Kb 472 Kb 16720 Ki 2023-01-22T11:02:53-08:00 6.26 0.00 0.00 46 2 21 25501 Kb 5427 Kb 218 Kb 472 Kb 16715 Ki &lt;...&gt; 2023-01-22T11:03:37-08:00 100.00 0.00 0.00 61 2 21 25874 Kb 5696 Kb 221 Kb 472 Kb 55235 Ki 2023-01-22T11:03:38-08:00 100.00 0.00 0.00 61 2 21 25955 Kb 5565 Kb 218 Kb 472 Kb 55139 Ki Code || QPS Time rand:uniform(). 8 61547 Ki 130 ns","ref":"readme.html#quick-start-command-line"},{"type":"extras","title":"Overview - Benchmark","doc":"Running benchmark is called a job , see erlperf_job for detailed description. Every job has a controller process, responsible for starting and stopping worker processes, or workers . Worker processes execute runner function in a tight loop, incrementing iteration counter. Benchmark runs either for a specified amount of time ( sample duration in continuous mode), or until requested number of iterations is made (timed mode). Resulting sample is total number of iterations for all workers, or elapsed time it took in timed mode. The process repeats until the specified amount of samples is collected, producing a report (see details below). For comparison convenience, basic reports contain QPS - historical metric from the original implementation (designed for network service throughput assessment). It is approximate amount of runner iterations per sample_duration achieved by all workers of the job . Given that default duration is 1 second, QPS is a good proxy for the total job throughput. Single worker performance can be estimated using time metric. It can also be considered as function latency - how long it takes on average to execute a single iteration of a runner . Benchmark definition A benchmark may define following functions: runner : code that is executed in the tight loop init (optional): executed once when the job starts done (optional): executed once when the job is about to stop init_runner (optional): executed on every worker process startup See erlperf_job for the detailed reference and ways to define a function ( callable ). Note that different ways to call a function have different performance characteristics: $ ./erlperf &#39;{rand, uniform, []}&#39; &#39;rand:uniform().&#39; -l 10M Code || QPS Time Rel rand:uniform(). 1 18519 Ki 54 ns 100% {rand,uniform,[]} 1 16667 Ki 60 ns 90% This difference may get more pronounced depending on ERTS version and runner code: ( erlperf @ max - au ) 7 &gt; erlperf : benchmark ( [ \#{ runner =&gt; &quot;runner(X) -&gt; is_float(X).&quot; , init_runner =&gt; &quot;2.&quot; } , \#{ runner =&gt; { erlang , is_float , [ 2 ] } } , \#{ runner =&gt; fun ( X ) -&gt; is_float ( X ) end , init_runner =&gt; &quot;2.&quot; } ] , \#{ } , undefined ) . [ 105824351 , 66424280 , 5057372 ] It is caused by the ERTS: running compiled code (first variant) with OTP 25 is two times faster than applying a function, and 20 times faster than repeatedly calling anonymous fun . Use the same invocation method to get a relevant result. Absolute benchmarking overhead may be significant for very fast functions taking just a few nanoseconds. Use timed mode for such occasions. Run options See erlperf module documentation and command line reference for all available options.","ref":"readme.html#benchmark"},{"type":"extras","title":"Overview - Benchmarking modes","doc":"Continuous mode Benchmarking is done by counting number of runner iterations done over a specified period of time ( sample_duration ). Two examples below demonstrate the effect caused by changing sample_duration . First run takes 20 samples ( -s 20 ) with 100 ms duration. Second invocation takes the same 20 sample, but with 200 ms duration ( -d 200 ). Note that all metrics, except a single iteration time, doubled. $ ./erlperf &#39;rand:uniform().&#39; -d 100 -s 20 Code || Samples Avg StdDev Median P99 Iteration rand:uniform(). 1 20 1647 Ki 0.39% 1648 Ki 1660 Ki 60 ns $ ./erlperf &#39;rand:uniform().&#39; -d 200 -s 20 Code || Samples Avg StdDev Median P99 Iteration rand:uniform(). 1 20 3354 Ki 0.16% 3354 Ki 3368 Ki 59 ns Timed mode In this mode runner code is executed for sample_duration iterations for every sample . Report contains average/median/p99 time it takes to produce a single sample. In the example below, it takes an average of 554 ms to make 10 million calls to rand:uniform/0 . $ ./erlperf &#39;rand:uniform().&#39; &#39;rand:uniform(1000).&#39; -l 10M -s 20 Code || Samples Avg StdDev Median P99 Iteration Rel rand:uniform(1000). 1 20 554 ms 0.37% 554 ms 563 ms 55 ns 100% rand:uniform(). 1 20 560 ms 0.60% 560 ms 564 ms 55 ns 99% Effectively, this example runs following code: loop(0) -&gt; ok; loop(Count) -&gt; rand:uniform(), loop(Count - 1). Timed mode has slightly less overhead compared to continuous mode. Timed mode does not support --concurrency setting, using only one process. However, it does support comparison run with multiple concurrent jobs. Concurrency estimation mode In this mode erlperf performs multiple continuous benchmarks with increasing concurrency. The test concludes when increasing worker count does not result in increase of the total throughput. Report contains statistics of the most successful run. This mode can also be used to detect bottlenecks, e.g. lock contention, single gen_server processes, or VM-wide shared resources ( persistent_term s). Example (with maximum concurrency limited to 50): $ ./erlperf &#39;{code, is_loaded, [local_udp]}&#39; -w 1 --max 50 -q Code || QPS Time {code,is_loaded,[local_udp]} 6 1665 Ki 3604 ns Same in the Erlang shell: &gt; erlperf : run ( { code , is_loaded , [ local_udp ] } , \#{ warmup =&gt; 1 } , \#{ max =&gt; 50 } ) . { 1676758 , 6 } In this example, 6 concurrent processes were able to squeeze 1676758 calls per second for code:is_loaded(local_udp) . In current OTP version code:is_loaded is implemented as a gen_server:call to a single process ( code_server ), that limits potential performance. See erlperf_job for the detailed description of different benchmarking modes.","ref":"readme.html#benchmarking-modes"},{"type":"extras","title":"Overview - Reports","doc":"Historically erlperf had only the basic reporting available for command line usage. Since 2.2 it is possible to request additional information. Basic report This is the default report form when less than 10 samples were collected. Use -r basic to force basic reports with 10 and more samples. Basic report contains following columns: Code : Erlang code supplied to the benchmark || : how many concurrent processes were running. In the timed mode, it is always 1. In the concurrency estimation mode, the number that achieved the highest total throughput (QPS) QPS : average number of runner code iterations (throughput). Measure per single sample_duration in the continuous mode. In the timed mode, calculated with the assumption that sample_duration is 1 second Time : single runner iteration time Rel : relative performance of this code, compared to others. Printed only when more than one runner is specified. Extended report When 10 or more samples were collected, this mode is the default. Use -r extended to force printing this report for smaller sample sets. Note that average, deviation, median and 99th percentile are calculated for the sample_duration . If you requested 20 samples of 100 ms in the continuous mode, these fields will contain iteration count per 100 ms. If you requested 10 million iterations ( -l 10M ), extended report for timed mode displays average time it takes to do 10M iterations. Single iteration time is printed as Iteration . Code, concurrency, and relative performance fields have the same meaning as in basic report. In addition, following columns are printed: Samples : how many samples were collected (useful when requesting continuous test with standard deviation requirement) Avg : same as QPS for continuous mode, but in the timed mode, average sample time StdDev : standard deviation from average Median : median value, in the continuous mode, median estimated throughput, in the timed mode - time to complete the requested iterations Iteration : single runner iteration time P99 : 99th percentile Full report This mode must be explicitly specified with -r full . Contains everything that extended report has. Includes extra information about the system used for benchmarking - OS type, CPU and Erlang VM characteristics.","ref":"readme.html#reports"},{"type":"extras","title":"Overview - Benchmarking compiled code","doc":"erlperf can be used to measure performance of your application running in production, or code that is stored on disk. Use -pa argument to add an extra code path. Example: $ ./erlperf &#39;args:parse([], \#{}).&#39; -pa _build/test/lib/argparse/ebin Code || QPS Time args:parse([], \#{}). 1 955 Ki 1047 ns If you need to add multiple released applications, supply ERL_LIBS environment variable instead: $ ERL_LIBS=&quot;_build/test/lib&quot; erlperf &#39;args:parse([], \#{}).&#39; Code || QPS Time args:parse([], \#{}). 1 735 Ki 1361 ns Usage in production It is possible to use erlperf to benchmark an application running in production. Add erlperf as a dependency, and use remote shell: # connect a remote shell to the production node erl -remsh production@max-au (production@max-au)3&gt; erlperf:run(timer, sleep, [1]). 488 Permanent continuous benchmarking You can run a job continuously, to examine performance gains or losses while doing hot code reload. This process is designed to help during development and testing stages, allowing to quickly notice performance regressions. Example source code: - module ( mymod ) . - export ( [ do / 1 ] ) . do ( Arg ) -&gt; timer : sleep ( Arg ) . Example below assumes you have erlperf application started (e.g. in a rebar3 shell ) % start a logger that prints VM monitoring information &gt; { ok , Logger } = erlperf_file_log : start_link ( ) . { ok , &lt; 0.235 . 0 &gt; } % start a job that will continuously benchmark mymod:do(), % with initial concurrency 2. &gt; JobPid = erlperf : start ( \#{ init_runner =&gt; &quot;rand:uniform(10).&quot; , runner =&gt; &quot;runner(Arg) -&gt; mymod:do(Arg).&quot; } , 2 ) . { ok , &lt; 0.291 . 0 &gt; } % increase concurrency to 4 &gt; erlperf_job : set_concurrency ( JobPid , 4 ) . ok . % watch your job performance % modify your application code, % set do(Arg) -&gt; timer:sleep(2*Arg), do hot code reload &gt; c ( mymod ) . { module , mymod } . % see that after hot code reload throughput halved!","ref":"readme.html#benchmarking-compiled-code"},{"type":"extras","title":"Overview - Timer precision","doc":"ERTS cannot guarantee precise timing when there is severe lock contention happening, and scheduler utilisation is 100%. This often happens with ETS: $ ./erlperf -c 50 &#39;ets:insert(ac_tab, {1, 2}).&#39; -d 100 -s 50 Timer accuracy problem detected, results may be inaccurate Code || Samples Avg StdDev Median P99 Iteration ets:insert(ac_tab, {1, 2}). 50 50 6079 82.27% 5497 40313 823 us Running 50 concurrent processes trying to overwrite the very same key of an ETS table leads to lock contention on a shared resource (ETS table/bucket lock). erlperf may detect this issue and switch to a busy wait loop for precise timing. This may result in lowered throughput and other metrics skew. erlperf does not attempt to pinpoint the source of contention, it is up to user to figure that out. It's recommended to use lock-counting emulator, or Linux perf utility to troubleshoot VM-level issues.","ref":"readme.html#timer-precision"},{"type":"extras","title":"Overview - Experimental features","doc":"These features are not fully supported. APIs may change in the future erlperf releases. Benchmarking in a cluster It's possible to run a job on a separate node in the cluster. See erlperf_cluster_monitor for additional details. % watch the entire cluster (printed to console) ( node1 @ host ) &gt; { ok , _ } = erlperf_history : start_link ( ) . { ok , &lt; 0.213 . 0 &gt; } ( node1 @ host ) &gt; { ok , ClusterLogger } = erlperf_cluster_monitor : start_link ( group_leader ( ) , 1000 , [ node , sched_util , jobs ] ) . { ok , &lt; 0.216 . 0 &gt; } % also log cluster-wide reports to file (jobs &amp; sched_util) ( node1 @ host ) &gt; { ok , FileLogger } = erlperf_cluster_monitor : start_link ( &quot;/tmp/cluster&quot; , 1000 , [ time , node , sched_util , jobs ] ) . { ok , &lt; 0.223 . 0 &gt; } % run the benchmarking process in a different node of your cluster ( node1 @ host ) &gt; rpc : call ( &#39;node2@host&#39; , erlperf , run , [ \#{ runner =&gt; { rand , uniform , [ ] } } ] ) .","ref":"readme.html#experimental-features"},{"type":"extras","title":"Command Line","doc":"Run erlperf with no arguments to get command line usage.","ref":"cli.html"},{"type":"extras","title":"Command Line - Synopsis","doc":"erlperf [FLAG] runner [INIT] [INIT_RUNNER] [DONE] [runner...]","ref":"cli.html#synopsis"},{"type":"extras","title":"Command Line - Flags","doc":"Short Long Description -c --concurrency Specifies the number of workers per job. Allowed only in continuous mode --cv Coefficient of variation. Accepted in continuous and concurrency estimation mode. Benchmark keeps running until standard deviation is below the specified number -i --isolation Requests to run every benchmark in a separate Erlang VM for isolation purposes -s --samples Number of samples to take. Defaults to 1 for timed mode, 3 for continuous and concurrency estimation -d --sample_duration Sample duration, in milliseconds, for continuous and concurrency estimation modes -l --loop Sample duration (iterations) for the timed mode. Engages timed mode when specified --max Maximum number of workers allowed in the concurrency estimation mode --min Starting number of workers in concurrency estimation mode -pa Adds extra code path to the Erlang VM. Useful for benchmarking *.beam files on your filesystem -r --report Requests basic , extended or full report. Defaults to basic when less than 10 samples are requested, and extended for 10 and more -q -squeeze Engages concurrency estimation mode -t -threshold Sets number of extra workers to try in concurrency estimation mode before concluding the test -v --verbose Turns on verbose mode, including generated source code, VM statistics and performance of continuous jobs) -w --warmup Warmup","ref":"cli.html#flags"},{"type":"extras","title":"Command Line - Benchmark code","doc":"At least one runner code is required. Specify multiple runner codes to perform a comparison run. Initialisation and cleanup definitions are read in the same order as runner codes. Example: # first runner receives 1 as input, second - 2 erlperf --init_runner &#39;1.&#39; &#39;run(1) -&gt; ok.&#39; &#39;run(2) -&gt; ok.&#39; --init_runner &#39;2.&#39; # next run fails with function_clause, because first runner receives &#39;2&#39;, and second - 1 erlperf --init_runner &#39;2.&#39; &#39;run(1) -&gt; ok.&#39; &#39;run(2) -&gt; ok.&#39; --init_runner &#39;1.&#39; Description --init Job initialisation code, see accepted callable formats below --init_runner Worker initialisation code --done Job cleanup code --init_all Default init code for all runners that do not have a specific code --init_runner_all Default init_runner code --done_all Default done code Accepted callable formats: valid Erlang code: timer:sleep(1). valid Erlang function: run() -&gt; timer:sleep(1). function with arguments: run(X) -&gt; timer:sleep(X).', 'run(X, Y) -&gt; timer:sleep(X), Y. tuple with module, function name and arguments: {timer, sleep, [1]} file name with call chain recording: record.trace . deprecated , do not use","ref":"cli.html#benchmark-code"},{"type":"extras","title":"Changelog","doc":"","ref":"changelog.html"},{"type":"extras","title":"Changelog - 2.2.2","doc":"added generated source code output in verbose mode","ref":"changelog.html#2-2-2"},{"type":"extras","title":"Changelog - 2.2.1","doc":"tested with OTP 26 and 27 updated to argparse 2.0.0","ref":"changelog.html#2-2-1"},{"type":"extras","title":"Changelog - 2.2.0","doc":"added extended and full reporting capabilities implemented additional statistics (standard deviation, median, p99) exported formatting APIs to allow escript-based benchmarks improved documentation, switched from edoc to ex_doc added convenience functions and defaults to monitor, file_log, cluster_monitor and history fixed cluster monitor output for multi-node configurations breaking change: consolidated monitor sample structure for cluster and local process groups fixed history store refined types for better Dialyzer analysis","ref":"changelog.html#2-2-0"},{"type":"extras","title":"Changelog - 2.1.0","doc":"fixed -w (--warmup) argument missing from command line synchronised worker startup when adding concurrency concurrent worker shutdown when reducing concurrency elevated job &amp; benchmark process priority to avoid result skew implemented scheduling problem detection (e.g. lock contention), added a busy loop method workaround","ref":"changelog.html#2-1-0"},{"type":"extras","title":"Changelog - 2.0.2","doc":"added convenience command line options: init_all, done_all, init_runner_all","ref":"changelog.html#2-0-2"},{"type":"extras","title":"Changelog - 2.0.1","doc":"minor bugfixes (friendlier error reporting)","ref":"changelog.html#2-0-1"},{"type":"extras","title":"Changelog - 2.0","doc":"incompatible change: erlperf requires runner arity to be defined explicitly. Code example: erlperf:run(\#{runner =&gt; {timer, sleep, []}, init_runner =&gt; &quot;1.&quot;}) , with erlperf making a guess that init_runner is defined, therefore its return value can be passed as the argument to timer:sleep/1 . This behaviour was confusing and is no longer supported. incompatible change: crashed runner causes entire job to stop (error contains the reason and stack trace) incompatible change: removed fprof/profiling support in favour of JIT + perf erlperf application is no longer required to be started for one-off benchmark runs","ref":"changelog.html#2-0"},{"type":"extras","title":"Changelog - 1.1.5:","doc":"support for OTP 25 (peer replacing slave)","ref":"changelog.html#1-1-5"},{"type":"extras","title":"Changelog - 1.1.4:","doc":"fixed an issue with pg already started moved profiling to spawned process","ref":"changelog.html#1-1-4"},{"type":"extras","title":"Changelog - 1.1.3:","doc":"addressed deprecation, updated to argparse 1.1.4","ref":"changelog.html#1-1-3"},{"type":"extras","title":"Changelog - 1.1.2:","doc":"updated command line parser to new argparse","ref":"changelog.html#1-1-2"},{"type":"extras","title":"Changelog - 1.1.1:","doc":"added support for OTP 24 added edoc documentation","ref":"changelog.html#1-1-1"},{"type":"extras","title":"Changelog - 1.0.0:","doc":"initial release","ref":"changelog.html#1-0-0"},{"type":"extras","title":"License","doc":"Copyright (c) 2019-2021 Maxim Fedorov maximfca@gmail.com All rights reserved. Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS &quot;AS IS&quot; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.","ref":"license.html"}]